==> 0.4.72.tar.gz: lib/interfaces/auth/mixins/firebase_otp_mixin.dart <==
import 'dart:async';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/foundation.dart';
import 'package:wowit/common.dart';
import 'package:wowit/interfaces/auth/auth.dart';
import 'package:wowit/interfaces/auth/mixins/fake_login_mixin.dart';
import 'package:wowit/interfaces/auth/mixins/firebase_login_mixin.dart';
import 'package:wowit/interfaces/auth/mixins/sql_login_mixin.dart';
import 'package:wowit/methods.dart';
import 'package:get/get.dart';
import 'package:wowit/interfaces/mixins/handle_exception_mixin.dart';
import 'package:wowit/message.dart';

mixin FirebaseOTPMixin on FirebaseLoginMixin, BaseAuth {
  String? _verificationId;

  Future verifyPhoneNumber(
    String phone, {
    required Function verifyOTP,
    required Function verificationFailed,
  }) async {
    final String phoneNumber = Methods.preparePhone(phone);
    Common.phoneNumberInput = phoneNumber;
    if (this is FakeLoginMixin) {
      if ((this as FakeLoginMixin).fakePhones.contains(phoneNumber)) {
        _verificationId = '123456';
        verifyOTP();
        return null;
      }
    }
    await FirebaseAuth.instance.verifyPhoneNumber(
      phoneNumber: phoneNumber,
      timeout: const Duration(seconds: 0),
      verificationCompleted: (authCredential) {},
      verificationFailed: (e) {
        _verificationId = null;
        verificationFailed();
        handelException(e, subTitle: 'verifyPhoneNumber');
      },
      codeSent: (verificationId, _) {
        _verificationId = verificationId;
        verifyOTP();
      },
      codeAutoRetrievalTimeout: (verificationId) {
        _verificationId = verificationId;
        verifyOTP();
      },
    );
  }

  Future<bool> singInWithOTP(
    String otp, {
    Function(dynamic e)? onError,
  }) async {
    if (_verificationId == null) return false;
    if (this is FakeLoginMixin) {
      bool isFakeLogin = otp == '123456' &&
          (this as FakeLoginMixin).fakePhones.contains(Common.phoneNumberInput);
      if (isFakeLogin) {
        if (kDebugMode) {
          print('================');
          print('|| fake login ||');
          print('================');
        }
        return true;
      }
    }
    var _credential = PhoneAuthProvider.credential(
      verificationId: _verificationId!,
      smsCode: otp,
    );
    try {
      await FirebaseAuth.instance.signInWithCredential(_credential);
      User? user = FirebaseAuth.instance.currentUser;
      if (user != null) {
        await initAccountFirebase(phone: user.phoneNumber!, uidPhone: user.uid);
        if (this is SQLLoginMixin) {
          await (this as SQLLoginMixin)
              .initAccountSQL(phone: user.phoneNumber!, uidPhone: user.uid);
        }
        return true;
      }
      return false;
    } catch (e) {
      if (onError != null) {
        onError(e);
      } else {
        handelException(e, subTitle: 'singInWithOTP');
      }
      return false;
    }
  }
}

mixin HandleExceptionOTPMixin on HandleExceptionMixin, FirebaseOTPMixin {
  final RegExp _errorCodeRegex = RegExp(r'(\[[a-zA-Z_\-\/]+\])');

  Map<String, Function> get errors => {
        'code-expired': theSMSCodeHasExpried,
        'invalid-verification-code': theSMSOTPInValid,
        'missing-verification-code': theSMSOTPMissing,
        'nvalid-phone-number': invalidPhone,
      };

  String? _getErrorCode(dynamic e) {
    String? error = _errorCodeRegex.stringMatch(e.toString());
    if (error == null || error.isEmpty) return null;
    return error.replaceAll(RegExp(r'\[|\]'), '').split('/').last;
  }

  void handleExceptionOTP(dynamic e) {
    String? error = _getErrorCode(e);
    if (error != null && errors.containsKey(error) && errors[error] != null) {
      errors[error]!.call();
    } else if (Common.supperAdmin || kDebugMode) {
      Get.snackbar(
        Message.getMessage('REGISTER_THONG_BAO'),
        e.toString(),
        snackPosition: SnackPosition.TOP,
        duration: const Duration(seconds: 5),
      );
    }
  }

  void theSMSOTPInValid() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_OPT_IN_VALID'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(seconds: 5),
    );
  }

  void theSMSCodeHasExpried() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_HAS_EXPIRED'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(seconds: 5),
    );
  }

  void theSMSOTPMissing() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_HAS_MISSING'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(seconds: 5),
    );
  }

  void invalidPhone() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_INVALID_PHONE'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(seconds: 5),
    );
  }
}

==> 0.5.6.tar.gz: lib/interfaces/auth/mixins/firebase_otp_mixin.dart <==
import 'dart:async';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/foundation.dart';
import 'package:wowit/common.dart';
import 'package:wowit/interfaces/auth/auth.dart';
import 'package:wowit/interfaces/auth/mixins/fake_login_mixin.dart';
import 'package:wowit/interfaces/auth/mixins/firebase_login_mixin.dart';
import 'package:wowit/interfaces/auth/mixins/sql_login_mixin.dart';
import 'package:wowit/methods.dart';
import 'package:get/get.dart';
import 'package:wowit/interfaces/mixins/handle_exception_mixin.dart';
import 'package:wowit/message.dart';

mixin FirebaseOTPMixin on FirebaseLoginMixin, BaseAuth {
  String? _verificationId;

  Future verifyPhoneNumber(
    String phone, {
    required Function verifyOTP,
    required Function verificationFailed,
  }) async {
    final String phoneNumber = Methods.preparePhone(phone);
    Common.phoneNumberInput = phoneNumber;
    if (this is FakeLoginMixin) {
      if ((this as FakeLoginMixin).fakePhones.contains(phoneNumber)) {
        _verificationId = '123456';
        verifyOTP();
        return null;
      }
    }
    await FirebaseAuth.instance.verifyPhoneNumber(
      phoneNumber: phoneNumber,
      timeout: const Duration(seconds: 0),
      verificationCompleted: (authCredential) {},
      verificationFailed: (e) {
        _verificationId = null;
        verificationFailed();
        handelException(e, subTitle: 'verifyPhoneNumber');
      },
      codeSent: (verificationId, _) {
        _verificationId = verificationId;
        verifyOTP();
      },
      codeAutoRetrievalTimeout: (verificationId) {
        _verificationId = verificationId;
        verifyOTP();
      },
    );
  }

  Future<bool> singInWithOTP(
    String otp, {
    Function(dynamic e)? onError,
  }) async {
    if (_verificationId == null) return false;
    if (this is FakeLoginMixin) {
      bool isFakeLogin = otp == '123456' &&
          (this as FakeLoginMixin).fakePhones.contains(Common.phoneNumberInput);
      if (isFakeLogin) {
        if (kDebugMode) {
          print('================');
          print('|| fake login ||');
          print('================');
        }
        return true;
      }
    }
    var credential = PhoneAuthProvider.credential(
      verificationId: _verificationId!,
      smsCode: otp,
    );
    try {
      await FirebaseAuth.instance.signInWithCredential(credential);
      User? user = FirebaseAuth.instance.currentUser;
      if (user != null) {
        await initAccountFirebase(phone: user.phoneNumber!, uidPhone: user.uid);
        if (this is SQLLoginMixin) {
          await (this as SQLLoginMixin)
              .initAccountSQL(phone: user.phoneNumber!, uidPhone: user.uid);
        }
        return true;
      }
      return false;
    } catch (e) {
      if (onError != null) {
        onError(e);
      } else {
        handelException(e, subTitle: 'singInWithOTP');
      }
      return false;
    }
  }
}

mixin HandleExceptionOTPMixin on HandleExceptionMixin, FirebaseOTPMixin {
  final RegExp _errorCodeRegex = RegExp(r'(\[[a-zA-Z_\-\/]+\])');

  Map<String, Function> get errors => {
        'code-expired': theSMSCodeHasExpried,
        'invalid-verification-code': theSMSOTPInValid,
        'missing-verification-code': theSMSOTPMissing,
        'nvalid-phone-number': invalidPhone,
      };

  String? _getErrorCode(dynamic e) {
    String? error = _errorCodeRegex.stringMatch(e.toString());
    if (error == null || error.isEmpty) return null;
    return error.replaceAll(RegExp(r'\[|\]'), '').split('/').last;
  }

  void handleExceptionOTP(dynamic e) {
    String? error = _getErrorCode(e);
    if (error != null && errors.containsKey(error) && errors[error] != null) {
      errors[error]!.call();
    } else if (Common.supperAdmin || kDebugMode) {
      Get.snackbar(
        Message.getMessage('REGISTER_THONG_BAO'),
        e.toString(),
        snackPosition: SnackPosition.TOP,
        duration: const Duration(seconds: 5),
      );
    }
  }

  void theSMSOTPInValid() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_OPT_IN_VALID'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(seconds: 5),
    );
  }

  void theSMSCodeHasExpried() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_HAS_EXPIRED'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(seconds: 5),
    );
  }

  void theSMSOTPMissing() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_HAS_MISSING'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(seconds: 5),
    );
  }

  void invalidPhone() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_INVALID_PHONE'),
      snackPosition: SnackPosition.TOP,
      duration: const Duration(seconds: 5),
    );
  }
}
